private DspCreative geminiAdToDspCreative(GeminiAd geminiAd) {
    DspCreative dspCreative = new DspCreative();
    CreativeDetails creativeDetails = new CreativeDetails();
    NativeDetails nativeDetails = new NativeDetails();
    NativeAssets nativeAssets = new NativeAssets();
    Set<NativeDetails> nativeDetailsSet = new HashSet<>();

    populateGeminiAdFromDspCreative(geminiAd, dspCreative);
    populateDSPImagesFromGeminiAd(geminiAd, images);
    creativeDetails.setImages(images);
    populateDSPDetailsFromGeminiAd(geminiAd, creativeDetails);
    populateDSPNativeDetailsFromGeminiAd(geminiAd, nativeDetails);
    populateDSPNativeAssetsFromGeminiAd(geminiAd, nativeAssets);

    // default value for DSP creative type
    dspCreative.setCreativeTypeId(CreativeType.NATIVE.getId());
    // default value for DSP mimetype
    dspCreative.setMimeTypeId(MIMEType.JPEG.getId());

    // Description field is mandatory in UI, default it to the creative name if it's missing
    if (StringUtils.isEmpty(nativeDetails.getDescription())){
        nativeDetails.setDescription(creative.getName());
    }

    // Styles feature is only supported for standard native display, standard video ads and carousel ads
    boolean stylesNotAllowedInDsp =
        !(CreativeTypeUtil.isNativeDisplay(dspCreative) || CreativeTypeUtil.isNativeVideo(dspCreative)) ||
        (!CreativeTypeUtil.isCarousel(dspCreative) && CreativeTypeUtil.isNonStandardNativeFormat(dspCreative));

    if (stylesNotAllowedInDsp) {
        creativeDetails.setStyles(null);
    }

    nativeDetails.setNativeAssets(nativeAssets);
    nativeDetailsSet.add(nativeDetails);
    creativeDetails.setNativeDetails(nativeDetailsSet);
    creativeDetails.setTranscodingNeededInBulk(true);
    dspCreative.setDetails(creativeDetails);

    populateDefaultValuesForDspCreative(dspCreative, geminiAd);
    // populate creative impression tracking
    Set<CreativeImpressionTracking> creativeImpressionsTrackingURLs = decorateImpressionTracking(geminiAd);
    dspCreative.setCreativeImpressionTracking(creativeImpressionsTrackingURLs);
    
    return dspCreative;
}

public void populateGeminiAdFields(GeminiAd geminiAd) {
    if (PreTapAdFormat.VIDEO_OPTOUT_MM == geminiAd.getEffectivePreTapAdFormat() || PreTapAdFormat.TEXT_OPTOUT_MM == geminiAd.getEffectivePreTapAdFormat()) {
        geminiAd.setOptOutOfMoments(Boolean.TRUE);
    } else {
        geminiAd.setOptOutOfMoments(Boolean.FALSE);
    }
}

private void populateDefaultValuesForDspCreative(DspCreative dspCreative, GeminiAd geminiAd) {
    // setting default values
    dspCreative.setAccountId(getDspAccountId());
    dspCreative.setAdBehaviorTypeId(AdBehaviorTypeEnum.UNCATEGORIZED.getId());
    dspCreative.setAdChoicesEnabled(true);
    dspCreative.setChannelType(ChannelType.DEFAULT.getId());
    dspCreative.setAllowAudit(true);
    dspCreative.setClientStatusId(ClientStatus.PLAY.getId());
    ExternalCreative externalCreative = new ExternalCreative();
    externalCreative.setSourceType(SourceType.GEMINI);
    externalCreative.setExternalCreativeId(String.valueOf(geminiAd.getId()));
    dspCreative.setExternalCreative(externalCreative);
    dspCreative.setCtStatus(CreativeCtStatusType.SUBMITTED.getId());
    //already reviewed and approved ad in Gemini
    dspCreative.setIsThirdPartyCodeSecure(true);
    dspCreative.setIsSmartAd(false);
    dspCreative.setIsDeepLink(false);
}

private void populateDSPNativeAssetsFromGeminiAd(GeminiAd geminiAd, NativeAssets nativeAssets) {
    nativeAssets.setImageSecuredUrl(geminiAd.getImageUrlHQ());
    nativeAssets.setLargeMobileImageSecuredUrl(geminiAd.getImageUrl());
}

private void populateDSPNativeDetailsFromGeminiAd(GeminiAd geminiAd, NativeDetails nativeDetails) {
    nativeDetails.setTitle(geminiAd.getTitle());
    nativeDetails.setDescription(geminiAd.getDescription());
    nativeDetails.setLandingPageUrl(geminiAd.getLandingUrl());
    nativeDetails.setLandingPageDomain(geminiAd.getDisplayUrl());
    if (StringUtils.isNotEmpty(geminiAd.getCallToActionText())) {
        nativeDetails.setCallToAction(geminiAd.getCallToActionText());
    }
    nativeDetails.setOptOutOfMoments(geminiAd.getOptOutOfMoments()); // field populated in func populateGeminiAdFields()
}

private void populateDSPDetailsFromGeminiAd(GeminiAd geminiAd, CreativeDetails creativeDetails) {
    creativeDetails.setCompanyName(geminiAd.getSponsoredBy());
    creativeDetails.setTitle(geminiAd.getTitle());
    creativeDetails.setDescription(geminiAd.getDescription());
    creativeDetails.setDisplayUrl(geminiAd.getDisplayUrl());
    creativeDetails.setOptOutOfMoments(geminiAd.getOptOutOfMoments()); // field populated in func populateGeminiAdFields()
    creativeDetails.setStyles(geminiAd.getStyles());
}

private void populateDSPImagesFromGeminiAd(GeminiAd geminiAd, CreativeImages images) {
    images.setImageSecuredUrl(geminiAd.getImageUrlHQ());
    images.setLargeMobileImageSecuredUrl(geminiAd.getImageUrl());
}

private void populateGeminiAdFromDspCreative(GeminiAd geminiAd, DspCreative creative) {
    creative.setName(geminiAd.getAdName());
    creative.setLandingPageUrl(geminiAd.getLandingUrl());
    creative.setLandingPageDomain(geminiAd.getDisplayUrl());
}

private Set<CreativeImpressionTracking> decorateImpressionTracking(GeminiAd geminiAd) {
    Set<CreativeImpressionTracking> creativeImpressionsTrackingURLs = new HashSet<>();
    if (CollectionUtils.isNotEmpty(geminiAd.getImpressionTrackingUrl())) {
        CreativeImpressionTracking creativeImpressionTracking = new CreativeImpressionTracking();
        creativeImpressionTracking.setImpTrackerUrl(geminiAd.getImpressionTrackingUrl()));
        creativeImpressionTracking.setTrackingEventTypeId(TrackingEventTypeEnum.impressionpixel.getId());
        creativeImpressionsTrackingURLs.add(creativeImpressionTracking);
    }
    if (StringUtils.isNotEmpty(geminiAd.getVideoCompletedViewTrackingURL())) {
        CreativeImpressionTracking videoCompletionImpressionTracking = new CreativeImpressionTracking();
        videoCompletionImpressionTracking.setImpTrackerUrl(geminiAd.getThirdPartyTrackingQuartile100Url()));
        videoCompletionImpressionTracking.setTrackingEventTypeId(TrackingEventTypeEnum.hundredpercentview.getId());
        creativeImpressionsTrackingURLs.add(videoCompletionImpressionTracking);
    }
    if (StringUtils.isNotEmpty(geminiAd.getVideoClickTrackingURL())) {
        CreativeImpressionTracking clickTracker = new CreativeImpressionTracking();
        clickTracker.setImpTrackerUrl(geminiAd.getThirdPartyTrackingClickUrl());
        clickTracker.setTrackingEventTypeId(TrackingEventTypeEnum.clicktrackingurl.getId());
        creativeImpressionsTrackingURLs.add(clickTracker);
    }
    return creativeImpressionsTrackingURLs;
}