public void populateDspCreativeFromGeminiAsset(DspCreative dspCreative, GeminiAdAsset geminiAdAsset) {
    CreativeDetails creativeDetails = dspCreative.getDetails();
    CreativeImages creativeDetails.getImages();
    NativeDetails nativeDetails = creativeDetails.getNativeDetails().iterator().next();
    NativeAssets nativeAssets = dnativeDetails.getNativeAssets();
    
    // populate fileds for primary video url
    String primaryVideoURL = extractVideoPrimaryUrl(geminiAdAsset, nativeDetails);
    if (StringUtils.isNotEmpty(primaryVideoURL)) {
        // populate creativeTypeId
        dspCreative.setCreativeTypeId(CreativeType.NATIVE_VIDEO.getId());
        // populate nativeAssets
        nativeAssets.setCreativeFileUrl(primaryVideoURL);
        nativeAssets.setYcsHttpsFileUrl(primaryVideoURL);
        // populate creativeImages
        images.setCreativeFileUrl(primaryVideoURL);
    }
    
    // populate supported ad assets
    if (geminiAdAsset.getType() == AssetType.IMAGE && geminiAdAsset.getUsageType() == AssetUsageType.IMAGE_THUMBNAIL) {
        // image thumbnail asset
        nativeAssets.setSmallImageSecuredUrl(geminiAdAsset.getMediaInfo().getOriginal().getUrl());
    } else if (geminiAdAsset.getType() == AssetType.IMAGE && geminiAdAsset.getUsageType() == AssetUsageType.IMAGE_PANORAMA) {
        // image panorama asset
        nativeAssets.setPanoramaImageSecuredUrl(geminiAdAsset.getMediaInfo().getOriginal().getUrl());
    } else if (isCarouselGeminiAdFormat(geminiAdAsset)) {
        // carousel asset
        populateCarouselAsset(dspCreative, geminiAdAsset);
    } else if (isTouchpointsGeminiAdFormat(geminiAdAsset)) {
        // touchpoint asset
        populateTouchpointsAsset(dspCreative, geminiAdAsset);
    } else if (isTilesGeminiAdFormat(geminiAdAsset)) {
        // tiles asset
        populateTilesAsset(dspCreative, geminiAdAsset);
    } else if (geminiAdAsset.getType() == AssetType.IMAGE && geminiAdAsset.getUsageType() == AssetUsageType.IMAGE_PORTRAIT
        // portrait image asset
        populateImagePortraitCreative(geminiAdAsset, creative);
    } else if (geminiAdAsset.getType() == AssetType.VIDEO && geminiAdAsset.getUsageType() == AssetUsageType.VIDEO_PORTRAIT) {
        // portrait video
        populateVideoPortraitCreative(geminiAdAsset, creative);
    } else if (geminiAdAsset.getType() == AssetType.IMAGE && geminiAdAsset.getUsageType() == AssetUsageType.IMAGE_PORTRAIT_BG) {
        // portrait bg
        creativeDetails.setBackgroundImageSecuredUrl(geminiAd.getImagePortraitUrlBG());
    } else if (StringUtils.isNotEmpty(geminiAdAsset.getMailAssetUrl())) {
        // mail sponsored
        populateMailSponsored(geminiAdAsset, creative);
    }
}

public String extractVideoPrimaryUrl(GeminiAdAsset geminiAdAsset, NativeDetails nativeDetails) {
    if (geminiAdAsset.getType() == AssetType.VIDEO && 
        (geminiAdAsset.getUsageType() == AssetUsageType.VIDEO_PRIMARY || adAsgeminiAdAssetset.getUsageType() == AssetUsageType.VIDEO_PORTRAIT)) {
        // populate video autoloop and return asset
        nativeDetails.setAllowVideoAutoloop(geminiAdAsset.getAutoloop());
        return geminiAdAsset.getMediaInfo().getOriginal().getUrl();
    } else if (geminiAdAsset.getType() == AssetType.HTML || geminiAdAsset.getUsageType() == AssetUsageType.MULTI_IMAGE) {
        // return video subAsset
        Set<SubAsset> subAssets = GeminiAdAsset.getSubAssets();
        if (CollectionUtils.isNotEmpty(subAssets)) {
            for (SubAsset subAsset : subAssets) {
                if (subAsset.getUsageType() == AssetUsageType.VIDEO_PRIMARY) {
                    return subAsset.getUrl();
                }
            }
        }
    }
}

public boolean isTouchpointsGeminiAdFormat(GeminiAdAsset geminiAdAsset) {
    GeminiAdAssetType matchType = geminiAdAsset.getType();
    if (matchType == null || !matchType.name().startsWith("TOUCHPOINT_")) {
        return false;
    }

    // TOUCHPOINT_HQ and TOUCHPOINT_LARGE do not support in UCAM
    if (matchType.equals(GeminiAdAssetType.TOUCHPOINT_HQ) || matchType.equals(GeminiAdAssetType.TOUCHPOINT_LARGE)) {
        return false;
    }

    GeminiAdAssetStyles styles = geminiAdAsset.getStyles();
    if (styles.getX() == null || styles.getY() == null || styles.getW() == null || styles.getH() == null) {
        return false;
    }

    GeminiAdAssetTemplateType templateType = geminiAdAsset.getTemplateType();
    if (!EnumUtils.isValidEnum(GeminiAdAssetTemplateType.class,templateType.toString())) {
        return false;
    }
    if (StringUtils.isEmpty(geminiAdAsset.getImageUrl()) && StringUtils.isEmpty(geminiAdAsset.getLandingUrl())) {
        return false;
    }
    return true;
}

public boolean isCarouselGeminiAdFormat(GeminiAdAsset geminiAdAsset) {
    GeminiAdAssetType type = geminiAdAsset.getType();
    if (type == null || !type.equals(GeminiAdAssetType.MULTI_IMAGE)) {
        return false;
    }
    if (StringUtils.isEmpty(geminiAdAsset.getImageUrl()) || StringUtils.isEmpty(geminiAdAsset.getImageUrlHQ())) {
        return false;
    }
    return true;
}

public boolean isTilesGeminiAdFormat(GeminiAdAsset geminiAdAsset) {
    GeminiAdAssetType matchType = geminiAdAsset.getType();
    return matchType != null && matchType.name().startsWith("TILES_"));
}
